/* -*- Mode: C; tab-width:4 -*- */
/* ex: set ts=4 shiftwidth=4 softtabstop=4 cindent: */
/**
 * @brief Example module for SOS
 * This module shows some of concepts in SOS
 */

/**
 * Module needs to include <module.h>
 */
//#undef _MODULE_

#include <sys_module.h>

//#define LED_DEBUG
#include <led_dbg.h>

#include "neighbor.h"

#include <MassWare/MassWare/massware.h>

#define NEIGHBOR_MEASURE_TID           0
#define STATUS_CHANGE_BASE             10
/**
 * Module can define its own state
 */
typedef struct {
  uint8_t pid;
  uint8_t state;
  int32_t sample_timer_interval; //default (1024*3 = 3s)
  MassOutputInterf * outfs;
  uint8_t outf_cnt;
  bool bstarted;
} app_state_t;

static int8_t neighbor_msg_handler(void *start, Message *e);

/**
 * This is the only global variable one can have.
 */
static const mod_header_t mod_header SOS_MODULE_HEADER = {
	.mod_id         = NEIGHBERRING_MOD_ID,
	.state_size     = sizeof(app_state_t),
	.num_sub_func   = 0,
	.num_prov_func  = 0,
	.platform_type  = HW_TYPE /* or PLATFORM_ANY */,
	.processor_type = MCU_TYPE,
	.code_id        = ehtons(NEIGHBERRING_MOD_ID),
	.module_handler = neighbor_msg_handler,
};


static int8_t neighbor_msg_handler(void *state, Message *msg)
{
  /**
   * The module is passed in a void* that contains its state.  For easy
   * reference it is handy to typecast this variable to be of the
   * applications state type.  Note that since we are running as a module,
   * this state is not accessible in the form of a global or static
   * variable.
   */
  app_state_t *s = (app_state_t*)state;

  /**
   * Switch to the correct message handler
   */
  switch (msg->type){
	/**
	 * MSG_INIT is used to initialize module state the first time the
	 * module is used.  Many modules set timers at this point, so that
	 * they will continue to receive periodic (or one shot) timer events.
	 */
  case MSG_INIT:
  {
	  s->pid = msg->did;
	  s->state = 0;
      s->sample_timer_interval = 2048; // 1024*2 = 2s
	  s->outfs = NULL;
	  s->outf_cnt = 0;
	  s->bstarted = false;

	  break;
	}

  /**
    * set the output component id (mod_id) and the interface of the component (msg_type)
	* so that masslets can be connected dynamically, used for arch reconfiguration
	*/    
  case GET_NEIGHBOR_NUM:
  {
      // PostDestMsg *pdmsg = (PostDestMsg*)sys_msg_take_data(msg);
	  PostDestMsg *pdmsg = (PostDestMsg*)(msg->data);
	  MassOutputInterf * outf = search_interface(s->outfs, GET_NEIGHBOR_NUM);
	  if (outf == NULL) {
		  outf = (MassOutputInterf*) sys_malloc(sizeof(MassOutputInterf));
		  outf->out_msg_type = GET_NEIGHBOR_NUM;
		  outf->dst_mod_id = pdmsg->dst_mod_id;
		  outf->dst_msg_type = pdmsg->msg_type;
		  outf->next = s->outfs;
		  s->outfs = outf;
		  s->outf_cnt ++;
	  }
	  else {
		  outf->dst_mod_id = pdmsg->dst_mod_id;
		  outf->dst_msg_type = pdmsg->msg_type;
	  }
	  break;
  }

  /**
    * set the parameters of this component, used for parameter tunning.
	*/    
  case SET_BEACON_FREQ:
  {
	s->sample_timer_interval = (int32_t)(ehtonl(*((uint32_t*)(msg->data))));
	if (s->bstarted) 
		sys_timer_restart(NEIGHBOR_MEASURE_TID, s->sample_timer_interval);
	break;
  }

  /**
    * Implement this module as a masslet. so that I can be activated by massware,
	* instead of SOS MSG_INIT,
	*/
  case START_MEASURING:
  {
	  /**
	   * The timer API takes the following parameters:
	   * - Timer ID (used to distinguish multiple timers of different
	   *   ..types on the same host)
	   * - Timer delay in
	   * - Type of timer
	   */
	  sys_timer_start(NEIGHBOR_MEASURE_TID, s->sample_timer_interval , TIMER_REPEAT);
	  s->bstarted = true;
	  break;
	}

	/**
	 * All timers addressed to this PID, regardless of the timer ID, are of
	 * type MSG_TIMER_TIMEOUT and handled by this handler.  Timers with
	 * different timer IDs can be further distinguished by testing for the
	 * type, as demonstrated in the relay module.
	 */
  case MSG_TIMER_TIMEOUT:
	{
	  MassOutputInterf * outf = search_interface(s->outfs, GET_NEIGHBOR_NUM);
	  if (s->state % STATUS_CHANGE_BASE == 0) {
		  sys_post_value(outf->dst_mod_id, outf->dst_msg_type, ehtonl((uint32_t)2), 0);
	  }
	  else if (s->state % STATUS_CHANGE_BASE == 5) {
		  sys_post_value(outf->dst_mod_id, outf->dst_msg_type, ehtonl((uint32_t)0), 0);
	  }

      s->state ++;
	  return SOS_OK;
	}


	/**
	 * MSG_FINAL is used to shut modules down.  Modules should release all
	 * resources at this time and take care of any final protocol
	 * shutdown.
	 */
  case MSG_FINAL:
	{
	  // release memory
	  release_interface(s->outfs);
	  s->outfs = NULL;
	  s->outf_cnt = 0;
	  s->bstarted = false;

	  /**
	   * Stop the timer
	   */
	  sys_timer_stop(NEIGHBOR_MEASURE_TID);

	  break;
	}

	/**
	 * The default handler is used to catch any messages that the module
	 * does no know how to handle.
	 */
	default:
	return -EINVAL;
  }

  /**
   * Return SOS_OK for those handlers that have successfully been handled.
   */
  return SOS_OK;
}

#ifndef _MODULE_
mod_header_ptr neighbor_get_header()
{
  return sos_get_header_address(mod_header);
}
#endif


